@@ -13,15 +13,15 @@ handles a number of common problems in multi-armed bandit problems, including
 contextual bandits, delayed reward, and restless bandits.
 
 This library is designed to be easy to use and extend. It is built on top of
-scikit-learn, and uses scikit-learn's estimators to model the arms. This
+scikit-learn, and uses scikit-learn-style estimators to model the arms. This
 allows you to use any scikit-learn estimator that supports the `partial_fit`
 and `sample` methods as an arm in a bandit. Restless bandits also require the
 `decay` method.
 
 This library is still under development, and the API is subject to change.
 
-Arm Class
-=========
+Bandit and Arm Classes
+======================
 
 The `Arm` class is the base class for all arms in a bandit. Its constructor
 takes two arguments, `action_function` and `reward_function`, which represent
@@ -31,19 +31,19 @@ the reward from the outcome of the action.
 .. autosummary::
     :toctree: _autosummary
 
+    Bandit
     Arm
 
 
-Bandit Class and Decorators
-===========================
+Bandit Decorators
+=================
 
-These class utilities can be used to create bandit algorithms from classes
-that define variables that implement the `Arm` protocol.
+These class decorators can be used to specialize `Bandit` subclasses for
+particular problems.
 
 .. autosummary::
     :toctree: _autosummary
 
-    Bandit
     contextual
     restless
 
@@ -64,7 +64,12 @@ Estimators
 ==========
 
 These estimators are the underlying models for the arms in a bandit. They
-should be passed to the `learner` argument of the `bandit` decorator.
+should be passed to the `learner` argument of the `bandit` decorator. Each
+of these Bayesian estimators can be converted to a recursive estimator by
+passing a `learning_rate` argument to the constructor that is less than 1.
+Each of them implement a `decay` method that uses the `learning_rate` to
+increase the variance of the prior. This is a type of state-space model that
+is useful for restless bandits.
 
 .. autosummary::
     :toctree: _autosummary
