@@ -1,3 +1,4 @@
+from __future__ import annotations
 from collections import defaultdict
 from functools import partial
 from typing import Any, Dict, Union, cast
@@ -12,6 +13,8 @@ from sklearn.utils.validation import check_X_y  # type: ignore
 from sklearn.utils.validation import NotFittedError, check_is_fitted
 from typing_extensions import Self
 
+from ._np_utils import groupby_array
+
 
 class DirichletClassifier(BaseEstimator, ClassifierMixin):  # type: ignore
     """
@@ -24,7 +27,8 @@ class DirichletClassifier(BaseEstimator, ClassifierMixin):  # type: ignore
         training data.
     learning_rate : float, default=1.0
         Learning rate for the Dirichlet distribution. Higher values will give
-        more weight to recent data.
+        more weight to recent data. This transforms the model into a recursive
+        Bayesian estimator.
     random_state : Union[int, np.random.Generator, None], default=None
         Random state for sampling from the Dirichlet distribution.
 
@@ -98,7 +102,7 @@ class DirichletClassifier(BaseEstimator, ClassifierMixin):  # type: ignore
         alphas: Dict[Union[int, str], float],
         *,
         learning_rate: float = 1.0,
-        random_state: Union[int, np.random.Generator, None] = None
+        random_state: Union[int, np.random.Generator, None] = None,
     ) -> None:
         self.alphas = alphas
         self.learning_rate = learning_rate
@@ -156,13 +160,8 @@ class DirichletClassifier(BaseEstimator, ClassifierMixin):  # type: ignore
         return self
 
     def _fit_helper(self, X: NDArray[Any], y: NDArray[Any]):
-        sort_keys = X[:, 0].argsort()  # type: ignore
-        X, y = X[sort_keys], y[sort_keys]  # type: ignore
-
-        groups, start_indexes = np.unique(X[:, 0], return_index=True)
-
-        for group, arr in zip(groups, np.split(y, start_indexes)[1:]):
-            key = group.item()
+        for group, arr in groupby_array(X[:, 0], y, by=X[:, 0]):
+            key = group[0].item()
             vals = np.row_stack((self.known_alphas_[key], arr))
 
             decay_idx = np.flip(np.arange(len(vals)))  # type: ignore
@@ -230,7 +229,8 @@ class GammaRegressor(BaseEstimator, RegressorMixin):
     beta : float
         Inverse scale parameter of the gamma distribution.
     learning_rate : float, default=1.0
-        Learning rate for the model.
+        Learning rate for the model. This transforms the model into a recursive
+        Bayesian estimator.
     random_state : int, np.random.Generator, default=None
         Random state for the model.
 
@@ -287,7 +287,7 @@ class GammaRegressor(BaseEstimator, RegressorMixin):
         beta: float,
         *,
         learning_rate: float = 1.0,
-        random_state: Union[int, np.random.Generator, None] = None
+        random_state: Union[int, np.random.Generator, None] = None,
     ) -> None:
         self.alpha = alpha
         self.beta = beta
@@ -330,22 +330,13 @@ class GammaRegressor(BaseEstimator, RegressorMixin):
     def _fit_helper(self, X: NDArray[Any], y: NDArray[Any]):
         # Performs the gamma-poisson update
 
-        # Sort the data by the first column
-        sort_keys = X[:, 0].argsort()  # type: ignore
-        X, y = X[sort_keys], y[sort_keys]  # type: ignore
-
-        # Get the unique groups and their start indexes
-        groups, start_indexes = np.unique(X[:, 0], return_index=True)
-
-        for group, arr in zip(groups, np.split(y, start_indexes)[1:]):
-            key = group.item()
+        for group, arr in groupby_array(X[:, 0], y, by=X[:, 0]):
+            key = group[0].item()
 
             # the update is computed by stacking the prior with the data,
             # where alpha and the counts are stacked together and beta is
             # stacked with ones
-
             data = np.column_stack((arr, np.ones_like(arr)))
-
             vals = np.row_stack((self.coef_[key], data))
 
             # Calculate the decay index for nonstationary models
@@ -425,9 +416,8 @@ class NormalRegressor(BaseEstimator, RegressorMixin):
     beta : float
         The prior for the precision of the noise.
     learning_rate : float, default=1.0
-        The learning rate for the model. If `learning_rate` is less than 1.0, the
-        model is nonstationary and the prior is decayed by a factor of
-        `learning_rate` at each iteration.
+        The learning rate for the model. This transforms the model into a recursive
+        Bayesian estimator, specifically a Kalman filter.
     random_state : int, np.random.Generator, default=None
         The random state for the model. If an int is passed, it is used to
         seed the numpy random number generator.
@@ -494,7 +484,7 @@ class NormalRegressor(BaseEstimator, RegressorMixin):
         beta: float,
         *,
         learning_rate: float = 1.0,
-        random_state: Union[int, np.random.Generator, None] = None
+        random_state: Union[int, np.random.Generator, None] = None,
     ) -> None:
         self.alpha = alpha
         self.beta = beta
@@ -648,6 +638,9 @@ class NormalInverseGammaRegressor(NormalRegressor):
         Prior shape parameter of the variance.
     b : float, default=0.1
         Prior rate parameter of the variance.
+    learning_rate : float, default=1.0
+        Learning rate for the model. This transforms the model into a
+        recursive Bayesian estimator, specifically a Kalman filter.
     random_state : int, np.random.Generator, or None, default=None
         Random state for the model.
 
@@ -722,7 +715,7 @@ class NormalInverseGammaRegressor(NormalRegressor):
         a=0.1,
         b=0.1,
         learning_rate=1.0,
-        random_state: Union[int, np.random.Generator, None] = None
+        random_state: Union[int, np.random.Generator, None] = None,
     ):
         self.mu = mu
         self.lam = lam
